/*  Begin aqua_surface_phill_ugauss.cpp  */

/*  "Aqua_Surface_Phillips" with unit gaussian distribution  */

/*
  Copyright (C) 2003  Jocelyn Frchot

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 of the License.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/****************  includes  ****************/


#include "aqua_surface_phill_ugauss.h"

#include "aqua_rng_ugaussian.h"
#include "aqua_surface_phillips.h"


/****************  public functions  ****************/


Aqua_Surface_Phill_Ugauss::
Aqua_Surface_Phill_Ugauss(int points_x,    /* numbers of discrete sample...   */
			  int points_z,    /* ...points, must be power of two */
			  float size_x,               /*  meters  */
			  float size_z,               /*  meters  */
			  float depth, /* of water, meters, 0.0 means infinite*/
			  float displacement_factor,  /*  horizontal  */
			  float spectrum_factor,
			  float smallest_wave,
			  float wind_speed,
			  float wind_angle,
			  float loop_time)   /*  seconds (0.0 means no loop)  */
  : Aqua_Surface_Phillips(points_x,
			  points_z,
			  size_x,
			  size_z,
			  depth,
			  displacement_factor,
			  spectrum_factor,
			  smallest_wave,
			  wind_speed,
			  wind_angle,
			  this->rng = new class Aqua_Rng_Ugaussian(),
			  loop_time)
{
  delete this->rng;
}


/*  End aqua_surface_phill_ugauss.cpp  */
